/**
 * Package: mb_metadata_edit
 *
 * Description:
 *
 * Files:
 *
 * SQL:
 * 
 * Help:
 *
 * Maintainer:
 * http://www.mapbender.org/User:Christoph_Baudson
 *
 * License:
 * Copyright (c) 2009, Open Source Geospatial Foundation
 * This program is dual licensed under the GNU General Public License
 * and Simplified BSD license.
 * http://svn.osgeo.org/mapbender/trunk/mapbender/license/license.txt
 */

var $metadataLayer = $(this);
var $metadataForm = $("<form>No layer selected.</form>").appendTo($metadataLayer);

var MetadataLayerApi = function (o) {
	var that = this;
	var validator;
	var formReady = false;
	var wmsId;
	var layerId;
	
	var disabledFields = [
		"layer_custom_category_id", 
		"layer_inspire_category_id", 
		"layer_md_topic_category_id", 
		"layer_keyword", 
		"layer_abstract", 
		"layer_title"
	];

	this.events = {
		initialized: new Mapbender.Event(),
		submit: new Mapbender.Event(),
		showOriginalLayerMetadata : new Mapbender.Event()
	};
		
	this.valid = function () {
		if (validator && validator.numberOfInvalids() > 0) {
			$metadataForm.valid();
			return false;
		}
		return true;
	};

	this.serialize = function (callback) {
		$metadataForm.submit();
		var data = null;
		if (this.valid()) {
			data = {
				layer: $metadataForm.easyform("serialize")
			};
		}
		if ($.isFunction(callback)) {
			callback(data);
		}
		return data !== null ? data.layer : data;
	};

	this.fillForm = function (obj) {
		$(disabledFields).each(function () {
			$("#" + this).removeAttr("disabled");
		});

		layerId = obj.layer_id;

		$metadataForm.easyform("reset");
		
		// get metadata from server
		var req = new Mapbender.Ajax.Request({
			url: "../plugins/mb_metadata_server.php",
			method: "getLayerMetadata",
			parameters: {
				"id": layerId
			},
			callback: function (obj, result, message) {
				if (!result) {
					return;
				}
				$metadataForm.easyform("fill", obj);
				that.valid();
			}
		});
		req.send();		
	};
	
	this.fill = function (obj) {
		$metadataForm.easyform("fill", obj);
	};
	
	var showOriginalLayerMetadata = function () {
		that.events.showOriginalLayerMetadata.trigger({
			data : {
				wmsId : wmsId,
				layerData : $metadataForm.easyform("serialize")
			}
		});
	};
	

	this.init = function (obj) {
		delete layerId;
		$metadataForm.easyform("reset");

		wmsId = obj;
		
		if (!wmsId) {
			return;
		}
		
		var formData = arguments.length >= 2 ? arguments[1] : undefined;

		if (!formReady) {
			$metadataForm.load("../plugins/mb_metadata_layer.php", function () {
				$metadataForm.find(".help-dialog").helpDialog();

				$metadataForm.find(".original-metadata-layer").bind("click", function() {
					showOriginalLayerMetadata();
				});	

				validator = $metadataForm.validate({
					submitHandler: function () {
						return false;
					}
				});

				that.events.initialized.trigger({
					wmsId: wmsId
				});
				formReady = true;
				
				// select layer in tree if set
				
				// init map
				
				// fill layer form if set
			});
			return;
		}
		$(disabledFields).each(function () {
			$("#" + this).attr("disabled", "disabled");
		});
		that.events.initialized.trigger({
			wmsId: wmsId
		});
	};
	
	Mapbender.events.localize.register(function () {
		that.valid();
		var formData = $metadataForm.easyform("serialize");
		formReady = false;
		that.init(wmsId, formData);
	});
	Mapbender.events.init.register(function () {
		that.valid();
	});
};

$metadataLayer.mapbender(new MetadataLayerApi(options));
