<?php
require_once dirname(__FILE__) . "/../../core/globalSettings.php";
require_once dirname(__FILE__) . "/../classes/class_user.php";
require_once dirname(__FILE__) . "/../classes/class_wms.php";

$ajaxResponse = new AjaxResponse($_POST);

function abort ($message) {
	global $ajaxResponse;
	$ajaxResponse->setSuccess(false);
	$ajaxResponse->setMessage($message);
	$ajaxResponse->send();
	die;
};

function getWms ($wmsId = null) {
	$user = new User(Mapbender::session()->get("mb_user_id"));
	$wmsIdArray = $user->getOwnedWms();
	if (!is_array($wmsIdArray) || count($wmsIdArray) === 0) {
		abort(_mb("No metadata sets available."));
	}
	if (!is_null($wmsId) && !in_array($wmsId, $wmsIdArray)) {
		abort(_mb("You are not allowed to access this WMS."));
	}
	return $wmsIdArray;
}

function getLayer ($layerId = null) {
	$user = new User(Mapbender::session()->get("mb_user_id"));
	$wmsIdArray = $user->getOwnedWms();
	if (!is_array($wmsIdArray) || count($wmsIdArray) === 0) {
		abort(_mb("No metadata sets available."));
	}
	$wmsId = wms::getWmsIdByLayerId($layerId);
	if (is_null($wmsId) || !in_array($wmsId, $wmsIdArray)) {
		abort(_mb("You are not allowed to access WMS " . $wmsId));
	}
	return;
}

switch ($ajaxResponse->getMethod()) {
	case "getWms" :
		$wmsIdArray = getWms();
		
		$wmsList = implode(",", $wmsIdArray);
		$sql = <<<SQL
	
SELECT wms.wms_id, wms.wms_title, wms.wms_timestamp, wms_version, m.status_comment
FROM wms LEFT JOIN mb_monitor AS m
ON wms.wms_id = m.fkey_wms_id 
WHERE wms_id IN ($wmsList);

SQL;
		$res = db_query($sql);
		$resultObj = array(
			"header" => array(
				"WMS ID",
				"Titel",
				"Timestamp",
				"Version",
				"Status"
			), 
			"data" => array()
		);

		while ($row = db_fetch_row($res)) {
			// convert NULL to '', NULL values cause datatables to crash
			$walk = array_walk($row, create_function('&$s', '$s=strval($s);'));
			$resultObj["data"][]= $row;
		}
		$ajaxResponse->setResult($resultObj);
		$ajaxResponse->setSuccess(true);
		break;

	case "getWmsMetadata" :
		$wmsId = $ajaxResponse->getParameter("id");
		getWms($wmsId);

		$wms = new wms();
		$wms->createObjFromDBNoGui($wmsId);

		$fields = array(
			"wms_id", 
			"wms_abstract", 
			"wms_title", 
			"fees", 
			"accessconstraints", 
			"contactperson", 
			"contactposition", 
			"contactvoicetelephone", 
			"contactfacsimiletelephone", 
			"contactorganization", 
			"address", 
			"city", 
			"stateorprovince", 
			"postcode", 
			"country", 
			"contactelectronicmailaddress",
			"wms_timestamp", 
			"wms_timestamp_create"
		);

		$resultObj = array();
		foreach ($fields as $field) {
			if ($field == "wms_timestamp" || $field == "wms_timestamp_create") {
				if ($wms->$field != "") {
					$resultObj[$field] = date('d.m.Y', $wms->$field);
					
				}
			}
			else {
				$resultObj[$field] = $wms->$field;	
			}
		}
		
		// layer searchable
		$resultObj["layer_searchable"] = array();
		foreach ($wms->objLayer as $layer) {
			if (intval($layer->layer_searchable) === 1) {
				$resultObj["layer_searchable"][] = intval($layer->layer_uid);
			}
		}
		
		$keywordSql = <<<SQL
	
SELECT DISTINCT keyword FROM keyword, layer_keyword 
WHERE keyword_id = fkey_keyword_id AND fkey_layer_id IN (
	SELECT layer_id from layer, wms 
	WHERE fkey_wms_id = wms_id AND wms_id = $wmsId
) ORDER BY keyword

SQL;

		$keywordRes = db_query($keywordSql);
		$keywords = array();
		while ($keywordRow = db_fetch_assoc($keywordRes)) {
			$keywords[]= $keywordRow["keyword"];
		}

		$resultObj["wms_keywords"] = implode(", ", $keywords);

		$termsofuseSql = <<<SQL
SELECT fkey_termsofuse_id FROM wms_termsofuse WHERE fkey_wms_id = $wmsId
SQL;

		$termsofuseRes = db_query($termsofuseSql);
		if ($termsofuseRes) {
			$termsofuseRow = db_fetch_assoc($termsofuseRes);
			$resultObj["wms_termsofuse"] = $termsofuseRow["fkey_termsofuse_id"];
		}
		else {
			$resultObj["wms_termsofuse"] = null;
		}
		

		$ajaxResponse->setResult($resultObj);
		$ajaxResponse->setSuccess(true);

		break;
	
	case "getLayerMetadata" :
		$layerId = $ajaxResponse->getParameter("id");
		getLayer($layerId);

		$sql = <<<SQL
	
SELECT layer_id, layer_name, layer_title, layer_abstract, layer_searchable 
FROM layer WHERE layer_id = $layerId;

SQL;
		$res = db_query($sql);

		$resultObj = array();
		while ($row = db_fetch_assoc($res)) {
			foreach ($row as $key => $value) {
				$resultObj[$key] = $value;
			}
		}

		$sql = <<<SQL
SELECT fkey_md_topic_category_id 
FROM layer_md_topic_category 
WHERE fkey_layer_id = $layerId
SQL;
		$res = db_query($sql);
		while ($row = db_fetch_assoc($res)) {
			$resultObj["layer_md_topic_category_id"][]= $row["fkey_md_topic_category_id"];
		}

		$sql = <<<SQL
SELECT fkey_inspire_category_id 
FROM layer_inspire_category 
WHERE fkey_layer_id = $layerId
SQL;
		$res = db_query($sql);
		while ($row = db_fetch_assoc($res)) {
			$resultObj["layer_inspire_category_id"][]= $row["fkey_inspire_category_id"];
		}

		$sql = <<<SQL
SELECT fkey_custom_category_id 
FROM layer_custom_category 
WHERE fkey_layer_id = $layerId
SQL;
		$res = db_query($sql);
		while ($row = db_fetch_assoc($res)) {
			$resultObj["layer_custom_category_id"][]= $row["fkey_custom_category_id"];
		}

		$sql = <<<SQL
SELECT keyword FROM keyword, layer_keyword 
WHERE keyword_id = fkey_keyword_id AND fkey_layer_id = $layerId
SQL;
		$res = db_query($sql);

		$resultObj["layer_keyword"] = array();
		while ($row = db_fetch_assoc($res)) {
			$resultObj["layer_keyword"][]= $row["keyword"];
		}

		$ajaxResponse->setResult($resultObj);
		$ajaxResponse->setSuccess(true);
		break;
	case "getLayerByWms" :
		$wmsId = $ajaxResponse->getParameter("id");
//		getWms($wmsId);

		$sql = <<<SQL
	
SELECT layer_id, layer_pos, layer_parent, layer_name, layer_title, layer_abstract, layer_searchable 
FROM layer WHERE fkey_wms_id = $wmsId ORDER BY layer_pos;

SQL;
		$res = db_query($sql);

		$rows = array();
		while ($row = db_fetch_assoc($res)) {
			$rows[] = $row;
		}
		$left = 1;

		function createNode ($left, $right, $row) {
			return array(
				"left" => $left,
				"right" => $right,
				"parent" => $row["layer_parent"] !== "" ? intval($row["layer_parent"]) : null,
				"pos" => intval($row["layer_pos"]),
				"attr" => array (
					"layer_id" => intval($row["layer_id"]),
					"layer_name" => $row["layer_name"],
					"layer_title" => $row["layer_title"],
					"layer_abstract" => $row["layer_abstract"],
					"layer_searchable" => intval($row["layer_searchable"])
				)
			);
		}

		function addSubTree ($rows, $i, $left) {
			$nodeArray = array();
			$addNewNode = true;
			for ($j = $i; $j < count($rows); $j++) {
				$row = $rows[$j];
				$pos = intval($row["layer_pos"]);
				$parent = $row["layer_parent"] !== "" ? intval($row["layer_parent"]) : null;
				
				// first node of subtree
				if ($addNewNode) {
					$nodeArray[]= createNode($left, null, $row);
					$addNewNode = false;
				}
				else {
					// new sub tree
					if ($parent === $nodeArray[count($nodeArray)-1]["pos"]) {
						$addedNodeArray = addSubTree($rows, $j, ++$left);
						
						$nodeArray[count($nodeArray)-1]["right"] = 
							$nodeArray[count($nodeArray)-1]["left"] + 
							2 * count($addedNodeArray) + 1;
						
						$left = $nodeArray[count($nodeArray)-1]["right"] + 1;

						$nodeArray = array_merge($nodeArray, $addedNodeArray);
						$j += count($addedNodeArray) - 1;
						$addNewNode = true;
						
					}
					// siblings
					elseif ($parent === $nodeArray[count($nodeArray)-1]["parent"]) {
						$nodeArray[count($nodeArray)-1]["right"] = ++$left;
						$nodeArray[]= createNode(++$left, null, $row);
					}
				}
			}
			if (is_null($nodeArray[count($nodeArray)-1]["right"])) {
				$nodeArray[count($nodeArray)-1]["right"] = ++$left;
			}
			return $nodeArray;
		}
		

		$nodeArray = addSubTree($rows, 0, 1);
		$resultObj = array(
			"nestedSets" => $nodeArray
		);
		
		$ajaxResponse->setResult($resultObj);
		$ajaxResponse->setSuccess(true);
		

		break;
	case "save":
		$data = $ajaxResponse->getParameter("data");
		
		try {
			$wmsId = intval($data->wms->wms_id);
		}
		catch (Exception $e) {
			$ajaxResponse->setSuccess(false);
			$ajaxResponse->setMessage(_mb("Invalid WMS ID."));
			$ajaxResponse->send();						
		}
		getWms($wmsId);
		
		$wms = new wms();
		$wms->createObjFromDBNoGui($wmsId);
		
		$columns = array(
			"wms_abstract", 
			"wms_title", 
			"fees", 
			"accessconstraints", 
			"contactperson", 
			"contactposition", 
			"contactvoicetelephone", 
			"contactfacsimiletelephone", 
			"contactorganization", 
			"address", 
			"city", 
			"stateorprovince", 
			"postcode", 
			"country", 
			"contactelectronicmailaddress",
			"wms_termsofuse"
		);
		foreach ($columns as $c) {
			$value = $data->wms->$c;
			if (!is_null($value)) {
				$wms->$c = $value;
			}
		}

		if (is_array($data->wms->layer_searchable)) {
			foreach ($wms->objLayer as &$layer) {
				$layer->layer_searchable = 0;
				for ($i = 0; $i < count($data->wms->layer_searchable); $i++) {
					$id = $data->wms->layer_searchable[$i];
					if ($id !== intval($layer->layer_uid)) {
						continue;						
					}
					$layer->layer_searchable = 1;
					break;
				}
				unset($id);
			}
		}

		try {
			$layerId = intval($data->layer->layer_id);
		}
		catch (Exception $e) {
		}
		
		if ($layerId) {
			$layer = $wms->getLayerById($layerId);
			if (!is_null($layer)) {
				$columns = array(
					"layer_abstract", 
					"layer_title",
					"layer_keyword",
					"layer_md_topic_category_id",
					"layer_inspire_category_id",
					"layer_custom_category_id"
				);			
	
				foreach ($columns as $c) {
					$value = $data->layer->$c;
					if ($c === "layer_keyword") {
						$layer->$c = explode(",", $value);
						foreach ($layer->$c as &$val) {
							$val = trim($val);
						}
					}
					elseif ($c === "layer_md_topic_category_id" 
						|| $c === "layer_inspire_category_id"
						|| $c === "layer_custom_category_id"
					) {
						if (!is_array($value)) {
							$layer->$c = array($value);
						}
						else {
							$layer->$c = $value;
						}
					}
					else {
						if (!is_null($value)) {
							$layer->$c = $value;
						}
					}
				}
			}
		}

		$wms->updateObjInDB($wmsId);

		
		$ajaxResponse->setMessage("Updated WMS metadata for ID " . $wmsId);
		$ajaxResponse->setSuccess(true);		
		
		break;
	default: 
		$ajaxResponse->setSuccess(false);
		$ajaxResponse->setMessage(_mb("An unknown error occured."));
		break;
}

$ajaxResponse->send();
?>