/**
 * Package: twitter_signin
 *
 * Description:
 * Authenticate with Twitter
 * 
 * Files:
 *  - http/plugins/mb_twitter_signin.js
 *
 * SQL:
 * > INSERT INTO gui_element(fkey_gui_id, e_id, e_pos, e_public, e_comment, 
 * > e_title, e_element, e_src, e_attributes, e_left, e_top, e_width, 
 * > e_height, e_z_index, e_more_styles, e_content, e_closetag, e_js_file, 
 * > e_mb_mod, e_target, e_requires, e_url) VALUES('<appId>','twitter_signin',
 * > 1,1,'Authenticate with Twitter','Authenticate with Twitter','img',
 * > '../img/twitter_signin.png','',900,10,NULL ,NULL ,NULL ,'','','',
 * > '../plugins/mb_twitter_signin.js','','','','');
 * >
 * > INSERT INTO gui_element_vars(fkey_gui_id, fkey_e_id, var_name, 
 * > var_value, context, var_type) VALUES('<appId>', 'twitter_signin', 
 * > 'consumerKey', '<your key>', '' ,'php_var');
 * >
 * > INSERT INTO gui_element_vars(fkey_gui_id, fkey_e_id, var_name, 
 * > var_value, context, var_type) VALUES('<appId>', 'twitter_signin', 
 * > 'consumerSecret', '<your secret>', 
 * > '' ,'php_var');
 * 
 * Help:
 * http://www.mapbender.org/Twitter_signin
 *
 * Maintainer:
 * http://www.mapbender.org/User:Christoph_Baudson
 * 
 * Parameters:
 * consumerKey      - your consumer key from Twitter (PHP var)
 * consumerSecret   - your consumer secret from Twitter (PHP var)
 *
 * License:
 * Copyright (c) 2009, Open Source Geospatial Foundation
 * This program is dual licensed under the GNU General Public License 
 * and Simplified BSD license.  
 * http://svn.osgeo.org/mapbender/trunk/mapbender/license/license.txt
 */

var $this = $(this);

var originalI18nObject = {
	"logIn" : "Log in at Twitter",
	"logOut" : "Log out at Twitter"
};

var translatedI18nObject = Mapbender.cloneObject(originalI18nObject);


var TwitterSignin = function (options) {
	var that = this;
	this.isSignedIn = function () {
		var req = new Mapbender.Ajax.Request({
			url: "../plugins/mb_twitter.php",
			method: "isSignedIn",
			parameters: {
				token: getOauthToken()
			},
			callback: function (obj, result, message) {
				if (!result) {
					alert(message);
					$this.text(translatedI18nObject.logIn);
					return;
				}
				if (obj.signedIn) {
					alert(message);
					$this.text(translatedI18nObject.logOut);
					$this.mapbender(function () {
						this.signedIn = true;
						this.events.signedIn.trigger();
					});
				}
				else {
					$this.text(translatedI18nObject.logIn);
					$this.mapbender(function () {
						this.signedIn = false;
						this.events.signedOut.trigger();
					});
				}
			}
		});
		req.send();
		
	};
	
	this.signedIn = false;

	this.events = {
		signedIn: new Mapbender.Event(),
		signedOut: new Mapbender.Event()
	};
	var getOauthToken = function () {
		var getString = document.location.search.substr(1,document.location.search.length);
		if (getString === '') {
			return null;
		}	
	
		var gArr = getString.split('&');
		for (i = 0; i < gArr.length; ++i) {
			var v = '';
			vArr = gArr[i].split('=');
	        	if (vArr.length > 1) {
				v = vArr[1];
			}
			var key = unescape(vArr[0]);
			if (key === "oauth_token") {
				return unescape(v);
			}
		}
		return null;
	};
	
	$this.click(function () {
		
		var method = that.signedIn ? "signout" : "signin";
		var req = new Mapbender.Ajax.Request({
			url: "../plugins/mb_twitter.php",
			method: method,
			parameters: {
			},
			callback: function (obj, result, message) {
				if (!result) {
					alert(message);
					return;
				}
				
				if (method === "signin" && obj.url && typeof obj.url === "string") {
				  window.location.href = obj.url;
				}
				else if (method === "signout") {
					var newUrl = window.location.href.replace(/oauth_token\=[0-9a-zA-Z]+/g, "");
					window.location.href = newUrl;
				
				}
			}
		});
		req.send();
	});	
	
};

$this.mapbender(new TwitterSignin());

Mapbender.events.init.register(function () {
	// check if signed in
	$this.mapbender(function () {
		this.isSignedIn();
	});
	
	Mapbender.events.localize.register(function () {
		Mapbender.modules.i18n.queue(options.id, originalI18nObj, function (translatedObj) {
			if (typeof translatedObj !== "object") {
				return;
			}
			translatedI18nObj = translatedObj;
			try {
				$this.mapbender(function () {
					this.isSignedIn();
				});
			}
			catch (exc) {
				new Mb_warning("Error when translating: " . exc.message);
			}
		});
	});	
});
