$.widget("mapbender.measure", {
	options: {
		opacity: 0.6,
		measurePointDiameter: 10,
		polygonFillSnapped: "#FC3",
		polygonFillDefault: "#FFF",
		polygonStrokeWidthDefault: 1,
		polygonStrokeWidthSnapped: 5,
		lineStrokeDefault: "#C9F",
		lineStrokeSnapped: "#F30",
		lineStrokeWidthDefault: 3,
		lineStrokeWidthSnapped: 5,
		pointFillSnapped: "#F90",
		pointFillDefault: "#CCF"
	},
	_measurePoints: [],
	_map: undefined,
	_srs: undefined,
	_currentDistance: 0,
	_totalDistance: 0,
	_polygonIsInvalid: false,
	_currentPolygonIsInvalid: false,
	_canvas: undefined,
	_isPolygon: function (pos) {

		var len = this._measurePoints.length;
		if (len < 2) {
			return false;
		}

		var p0 = this._measurePoints[0].pos;
		var pn = this._measurePoints[len - 1].pos;

		for (var i = 0; i < len - 1; i++)  {
			var pi = this._measurePoints[i].pos;
			var pj = this._measurePoints[i + 1].pos;

			if (i > 0 && this._lineIntersect(
				pi.x, pi.y, pj.x, pj.y, 
				p0.x, p0.y, pos.x, pos.y)
			) {
				return false;
			}
			if (this._lineIntersect(
				pi.x, pi.y, pj.x, pj.y, 
				pn.x, pn.y, pos.x, pos.y)
			) {
				this._currentPolygonIsInvalid = true;
				return false;
			}
		}
		this._currentPolygonIsInvalid = false;
		return true;
	},
	_lineIntersect: function ( x1, y1, x2, y2, x3, y3, x4, y4 ) {
		var isOnSegment = function (xi, yi, xj, yj, xk, yk) {
			// changed <= to < so the segments are allowed to touch!
			 return (xi < xk || xj < xk) && 
					(xk < xi || xk < xj) &&
			        (yi < yk || yj < yk) && 
					(yk < yi || xk < yj);
		};
		
		var computeDirection = function (xi, yi, xj, yj, xk, yk) {
			var a = (xk - xi) * (yj - yi);
			var b = (xj - xi) * (yk - yi);
			return a < b ? -1 : a > b ? 1 : 0;
		};
		
		var e1 = computeDirection(x3, y3, x4, y4, x1, y1);
		var e2 = computeDirection(x3, y3, x4, y4, x2, y2);
		var e3 = computeDirection(x1, y1, x2, y2, x3, y3);
		var e4 = computeDirection(x1, y1, x2, y2, x4, y4);
		return (((e1 > 0 && e2 < 0) || (e1 < 0 && e2 > 0)) &&
			((e3 > 0 && e4 < 0) || (e3 < 0 && e4 > 0))) ||
			(e1 === 0 && isOnSegment(x3, y3, x4, y4, x1, y1)) ||
			(e2 === 0 && isOnSegment(x3, y3, x4, y4, x2, y2)) ||
			(e3 === 0 && isOnSegment(x1, y1, x2, y2, x3, y3)) ||
			(e4 === 0 && isOnSegment(x1, y1, x2, y2, x4, y4));
	},
	_toRad: function (deg) {
		return deg * Math.PI/180;
	},
	//
	// calculate area
	//
	_calculateAreaMetric: function (pos) {
		if (this._measurePoints.length < 2) {
			return null;
		}
		
		var area = 0;
		var p0 = this._measurePoints[0].pos, pi, pj;
		for (var i = 0; i < this._measurePoints.length - 1; i++)  {
			pi = this._measurePoints[i].pos;
			pj = this._measurePoints[i + 1].pos;
			area += (pi.y + pj.y) * (pi.x - pj.x) / 2;
		}
		area += (p0.y + pj.y) * (p0.x - pj.x) / 2;
		return Math.abs(area);
	},
	_calculateAreaGeographic: function (pos) {
		if (this._measurePoints.length < 2) {
			return null;
		}
		
		// add current point and first point
		this._measurePoints.push({
			pos: pos
		});
		this._measurePoints.push(this._measurePoints[0]);
		
        var area = 0.0;
        var p1, p2;
        for(var i=0; i<this._measurePoints.length-1; i++) {
            p1 = this._measurePoints[i].pos;
            p2 = this._measurePoints[i+1].pos;
			var c = this._toRad(p2.x - p1.x) *
                    (2 + Math.sin(toRad(p1.y)) +
                    Math.sin(toRad(p2.y)));
            area += c;
        }
        area = area * 6378137.0 * 6378137.0 / 2.0;

		// remove current point and first point
		this._measurePoints.pop();
		this._measurePoints.pop();

		return Math.abs(area);
	},
	_calculateArea: function (pos) {
		switch (this._map.getSrs()) {
			case "EPSG:4326":
				return this._calculateAreaGeographic(pos);
			default:	
				return this._calculateAreaMetric(pos);
		}
		return null;
	},
	//
	// calculate distance
	//
	_calculateDistanceGeographic: function (a, b) {
		var lon_from = this._toRad(a.x);
		var lat_from = this._toRad(a.y);
		var lon_to = this._toRad(b.x);
		var lat_to = this._toRad(b.y);
		return Math.abs(6371229 * Math.acos(
			Math.sin(lat_from) * Math.sin(lat_to) + 
			Math.cos(lat_from) * Math.cos(lat_to) * 
			Math.cos(lon_from - lon_to)
		));
	},
	_calculateDistanceMetric: function (a, b) {
		return Math.abs(Math.sqrt(
			Math.pow(Math.abs(b.x - a.x), 2) + 
			Math.pow(Math.abs(b.y - a.y), 2)
		));
	},
	_calculateDistance: function (a, b) {
		if (a !== null && b !== null) {
			switch (this._map.getSrs()) {
				case "EPSG:4326":
					return this._calculateDistanceGeographic(a, b);
				default:	
					return this._calculateDistanceMetric(a, b);
			}
		}
		return null;
	},
	_isPointSnapped: function (p1, p2) {
		return p1.dist(p2) <= this.options.measurePointDiameter/2;
	},
	_isFirstPointSnapped: function (p) {
		if (this._measurePoints.length > 0 ) {
			var pos0 = this._measurePoints[0].mousePos;
			if (this._measurePoints.length > 2 && this._isPointSnapped(pos0, p)) {
				return true;
			}
		}
		return false;
	},
	_isLastPointSnapped: function (p) {
		if (this._measurePoints.length > 0 ) {
			var posn = this._measurePoints[this._measurePoints.length - 1].mousePos;
			if (this._measurePoints.length > 1 && this._isPointSnapped(posn, p)) {
				return true;
			}
		}
		return false;
	},
	_draw: function (pos, drawOptions) {
		this._canvas.clear();
		
		var str_path = "";

		if (!drawOptions.highlightFirst) {
			this._measurePoints.push(pos);
		}

		var len = this._measurePoints.length;
		if (len > 0) {
			for (var k=0; k < len; k++) {
				if (drawOptions.highlightLast && k === len - 1) {
					continue;
				}

				var pk = this._measurePoints[k].pos;
				var q = this._measurePoints[k].mousePos;

				str_path += (k === 0) ? 'M' : 'L';
				str_path += q.x + ' ' + q.y;
				
				if (drawOptions.drawPoints && 
					(k === 0 && !this._polygonIsInvalid || k >= len - 2)) {

					var circle = this._canvas.circle(q.x, q.y, this.options.measurePointDiameter);
					
					if (k === 0) {
						circle.attr({
							fill: drawOptions.highlightFirst ? 
								this.options.pointFillSnapped : this.options.pointFillDefault,
							stroke: "none",
							opacity: this.options.opacity
						});
					}
					else {
						circle.attr({
							fill: drawOptions.highlightLast && k === len - 2 ? 
								this.options.pointFillSnapped : this.options.pointFillDefault,
							stroke: "none",
							opacity: this.options.opacity
						});
					}
				}
			}
		}
		if (!drawOptions.highlightFirst) {
			this._measurePoints.pop();
		}
		
	
		if (this._isPolygon(this._measurePoints, pos) && !drawOptions.highlightLast && !this.polygonIsInvalid) {
			var poly = this._canvas.path(str_path + 'Z');
			poly.attr({
				fill: drawOptions.highlightFirst ? 
					this.options.polygonFillSnapped : this.options.polygonFillDefault, 
				stroke: drawOptions.highlightFirst || drawOptions.highlightLast ? 
					this.options.lineStrokeSnapped: this.options.lineStrokeDefault, 
				"stroke-width": drawOptions.highlightFirst ? 
					this.options.polygonStrokeWidthSnapped : this.options.polygonStrokeWidthDefault, 
				opacity: this.options.opacity
			});
		}

		var line = this._canvas.path(str_path);
		line.attr({
			stroke: drawOptions.highlightFirst || drawOptions.highlightLast ? 
				this.options.lineStrokeSnapped : this.options.lineStrokeDefault, 
			"stroke-width": drawOptions.highlightLast ? 
				this.options.lineStrokeWidthSnapped : this.options.lineStrokeWidthDefault, 
			opacity: this.options.opacity
		});
		line.toFront();   
	},
	_measure: function (e) {
		var self = $(this).data("measure");

		if (self._srs !== self._map.getSrs()){
//			button.stop();
			return;
		}

		var mousePos = self._map.getMousePosition(e);

		
		var firstPointSnapped = self._isFirstPointSnapped(mousePos) 
			&& !self._polygonIsInvalid;
		var lastPointSnapped = self._isLastPointSnapped(mousePos);


		var pos = {
			mousePos: mousePos,
			pos: self._firstPointSnapped ? 
				self._measurePoints[0].pos : lastPointSnapped ? 
					self._measurePoints[self._measurePoints.length - 1].pos : 
					self._map.convertPixelToReal(mousePos)
		};

		self._trigger("onmeasure", null, pos);
		
		var previousPoint = self._measurePoints.length > 0 ? 
			self._measurePoints[self._measurePoints.length - 1].pos : null;

		self._currentDistance = self._calculateDistance(
			previousPoint, 
			pos.pos
		);

		if (self._isPolygon(self._measurePoints, pos) && !self._polygonIsInvalid) {
//			updateArea(self._calculateArea(pos));
		}
		else {
//			measureDialog.find(".mb-measure-area").parent().hide();		
		}

		self._draw(pos, {
			highlightFirst: firstPointSnapped, 
			highlightLast: lastPointSnapped,
			drawPoints: true
		});		
	},
	reinitialize: function (e) {
		var self = $(this).data("measure");
		self._canvas.clear();
		self._measurePoints = [];
			
//			hideMeasureData();
			
		self._polygonIsInvalid = false;
		self._currentPolygonIsInvalid = false;
		self._totalDistance = 0;
			
		self.element
			.bind("click", self._addPoint)
			.bind("mousemove", self._measure)
			.unbind("click", self._reinitialize)
			.css("cursor", "crosshair");
		return false;
	},
	_addLastPoint: function (e) {
		this.element.unbind("click", this._addPoint)
			.unbind("mousemove", this._measure)
			.css("cursor", "auto")
			.bind("click", this._reinitialize);
	},	
	_addPoint: function (e) {
		var self = $(this).data("measure");

		var mousePos = self._map.getMousePosition(e);

		var pos = {
			mousePos: mousePos,
			pos: self._map.convertPixelToReal(mousePos),
			distance: self._currentDistance
		};

		self._trigger("pointadded", null, pos);
		
		var firstPointSnapped = self._isFirstPointSnapped(mousePos);
		var lastPointSnapped = self._isLastPointSnapped(mousePos);
		
		self._isPolygon(self._measurePoints, pos);
		if (self._currentPolygonIsInvalid) {
			self._polygonIsInvalid = true;
		}
		self._currentPolygonIsInvalid = false;

		if (lastPointSnapped || firstPointSnapped) {
			self._draw(pos, {
				highlightFirst: firstPointSnapped, 
				highlightLast: lastPointSnapped,
				drawPoints: false
			});
			self._addLastPoint(e);
		}
		else {
			self._measurePoints.push(pos);

			self._totalDistance += self._currentDistance;
			self._currentDistance = 0;
			
			lastPointSnapped = self._isLastPointSnapped(pos.mousePos);
			self._draw(pos, {
				highlightFirst: firstPointSnapped, 
				highlightLast: lastPointSnapped,
				drawPoints: true
			});
		}
		return true;
	},
	_init: function () {
		console.log(this.element);
	},
	_create: function () {
		// ":maps" is a Mapbender selector which 
		// checks if an element is a Mapbender map
		this.element = this.element.filter(":maps");

		if (!this.element.jquery || this.element.size() === 0) {
			$.error("This widget must be applied to a Mapbender map.");
		}
		
		this.element
			.bind("click", this._addPoint)
			.bind("mousemove", this._measure)
			.css("cursor", "crosshair");		

		this._map = this.element.mapbender();
		this._srs = this._map.getSrs();
		
		var $canvas = $("<div />").css({
			"z-index": 100,
			"position": "relative"
		}).appendTo(this.element);
		this._canvas = Raphael($canvas.get(0), this._map.getWidth(), this._map.getHeight());
	},
	destroy: function () {
		this.element
			.unbind("click", this._addPoint)
			.unbind("mousemove", this._measure)
			.css("cursor", "default");		

		this._canvas.clear();

		$.Widget.prototype.destroy.apply(this, arguments); // default destroy
	}
});
