<?php
require_once 'PHPUnit/Framework.php';
require_once dirname(__FILE__) . "/../../../lib/class_GetApi.php";

class GetApiTest extends PHPUnit_Framework_TestCase {

	public function testSingleLayer () {
		parse_str("LAYER=12", $getArray);

		$apiObject = new GetApi($getArray);
		
		$expected = array(
			array(
				"id" => 12
			)
		);
		$this->assertEquals($expected, $apiObject->getLayers());
	}

	public function testSingleLayerFromApplication () {
		parse_str("LAYER[application]=gui1&LAYER[id]=12", $getArray);
		$apiObject = new GetApi($getArray);
		
		$expected = array(
			array(
				"id" => 12,
				"application" => "gui1"
			)
		);
		$this->assertEquals($expected, $apiObject->getLayers());
	}

	public function testMultipleLayer () {
		parse_str("LAYER[0][application]=gui1&LAYER[0][id]=12&LAYER[1]=13", $getArray);
		$apiObject = new GetApi($getArray);
		
		$expected = array(
			array(
				"id" => 12,
				"application" => "gui1"
			),
			array(
				"id" => 13
			)
		);
		$this->assertEquals($expected, $apiObject->getLayers());
	}

	public function testMultipleLayerComplex () {
		parse_str("LAYER[visible]=false&LAYER[zoom]=true&LAYER[application]=gui1&LAYER[id]=12", $getArray);
		$apiObject = new GetApi($getArray);
		
		$expected = array(
			array(
				"id" => 12,
				"application" => "gui1",
				"visible" => false,
				"zoom" => true
			)
		);
		$this->assertEquals($expected, $apiObject->getLayers());
	}
	
	public function testSingleFeaturetype () {
		parse_str("FEATURETYPE=12", $getArray);
		$apiObject = new GetApi($getArray);
		
		$expected = array(
			array(
				"id" => 12
			)
		);
		$this->assertEquals($expected, $apiObject->getFeaturetypes());
	}
	
	public function testMultipleFeaturetypes () {
		parse_str("FEATURETYPE=12,13", $getArray);
		$apiObject = new GetApi($getArray);
		
		$expected = array(
			array(
				"id" => 12
			),
			array(
				"id" => 13
			)
		);
		$this->assertEquals($expected, $apiObject->getFeaturetypes());
	}

	public function testMultipleFeaturetypesArray () {
		parse_str("FEATURETYPE[]=12&FEATURETYPE[]=13", $getArray);
		$apiObject = new GetApi($getArray);
		
		$expected = array(
			array(
				"id" => 12
			),
			array(
				"id" => 13
			)
		);
		$this->assertEquals($expected, $apiObject->getFeaturetypes());
	}

	public function testMultipleFeaturetypesArrayComplex () {
		parse_str("FEATURETYPE[active]=false&FEATURETYPE[search][firstname]=a&FEATURETYPE[search][lastname]=b&FEATURETYPE[id]=12", $getArray);
		$apiObject = new GetApi($getArray);
		
		$expected = array(
			array(
				"id" => 12,
				"active" => false,
				"search" => array(
					"firstname" => "a",
					"lastname" => "b"
				)
			)
		);
		$this->assertEquals($expected, $apiObject->getFeaturetypes());
	}

}
?>